<?php

namespace FiloBlu\ExtAmastyXnotif\Helper;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\ProductAlert\Block\Product\View;
use Magento\Framework\Serialize\SerializerInterface;

class Data extends \Amasty\Xnotif\Helper\Data
{

    protected $_scopeConfig;
    protected $_serializer;

    public function __construct(
        \Magento\Framework\Registry $registry,
        \Magento\Framework\View\Element\BlockFactory $blockFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \Magento\Framework\App\Helper\Context $context,
        \Amasty\Xnotif\Helper\Config $config,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\App\Config\ScopeConfigInterface $_scopeConfig,
        \Magento\Framework\Serialize\SerializerInterface $_serializer
    ){
        parent::__construct($registry,$blockFactory,$stockRegistry,$context,$config,$storeManager);

        $this->_serializer = $_serializer;
        $this->_scopeConfig = $_scopeConfig;
        $this->registry = $registry;
    }

    public function getConfig() {
        $value = $this->_scopeConfig->getValue("amxnotif/attribute_in_grid/attribute");
        if ($value){
            return $this->_serializer->unserialize($value);
        }
        return "";
    }

    /* @return bool */
    public function isNewsletterSubscriptionEnabled(){
        return (bool)$this->getModuleConfig('outofstock_newsletter_subscription/enabled');
    }

    public function getNewsletterPrivacyText()
    {
        return $this->getModuleConfig('outofstock_newsletter_subscription/text');
    }

    /**
     * @param $path
     * @param int $storeId
     * @return mixed
     */
    public function getModuleConfig($path, $storeId = null)
    {
        return $this->_scopeConfig->getValue(
            'amxnotif/' . $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @param ProductInterface $product
     * @param View $alertBlock
     *
     * @return string
     */
    public function observeStockAlertBlock(ProductInterface $product, View $alertBlock)
    {
        $html = '';
        $currentProduct = $this->registry->registry('current_product');
        if (!$product->getId() || !$currentProduct) {
            return $html;
        }

        /*check if it is child product for replace product registered to child product.*/
        $isChildProduct = ($currentProduct->getId() != $product->getId());
        if ($isChildProduct) {
            $alertBlock->setData('parent_product_id', $currentProduct->getId());
            $alertBlock->setOriginalProduct($product);
        }
        $alertBlock->setSignupUrl($this->getSignupUrl(
            'stock',
            $product->getId(),
            $alertBlock->getData('parent_product_id')
        ));

        if ($alertBlock && !$product->getData('amxnotif_hide_alert')) {
            if (!$this->isLoggedIn()) {
                $alertBlock->setTemplate('FiloBlu_ExtAmastyXnotif::product/view_email.phtml');
            }

            $alertBlock->setData('amxnotif_observer_triggered', 1);
            $html = $alertBlock->toHtml();
            $alertBlock->setData('amxnotif_observer_triggered', null);
        }

        return $html;
    }

}